# Elevate
# Copyright 2007 Jeff Armstrong (jba@member.fsf.org) 
# 
# This program is free software; you can redistribute it and/or modify 
# it under the terms of the GNU General Public License as published by 
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version. 
#  
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License 
# along with this program; if not, write to:
# Free Software Foundation, Inc.
# 51 Franklin Street, Fifth Floor
# Boston, MA  02110-1301, USA

import pygame
from screen import *
from data import *

ENTERING = 1
WAITING = 2
ANGRYWAITING = 3
RIDING = 4
LEAVING = 5
DONE = 6

ANGRY_THRESHOLD = 65
ANGRY_LEAVING = 100

MAX_START_PATIENCE = 25
MAX_PATIENCE_RATE = 0.4
MIN_PATIENCE_RATE = 0.05

PASS_HEIGHT = int(0.80*getUnscaledFloorHeight())
PASS_WIDTH = int(0.6*PASS_HEIGHT)

BASE_PASS_LINE_WIDTH = 2.0

PASS_FONT_NAME = "CAB.TTF"

XMETER_SIZE = 0.15
YMETER_SIZE = 0.35

class Passenger(pygame.sprite.Sprite):

    rxpos = 0.0
    
    def __init__(self, floor, floor_tgt, patience, patience_rate):
        pygame.sprite.Sprite.__init__(self)

        # The floor this person starts on
        self.floor = floor
        self.tgt = floor_tgt

        # Patience level and rate at which it rises
        self.patience = patience
        self.p_rate = patience_rate

        # Walking status
        self.status = ENTERING
        self.legpos = [0,0]
        
        self.color = Color('Black')
        
        # Initialize the frame count
        self.framecount = 0

        # Other necessities
        self.rect = Rect(0,calculateFloorY(floor), \
                         int(getScaling()*PASS_WIDTH), \
                         int(getScaling()*PASS_HEIGHT))

        self.createSurface()
        
        self.font = pygame.font.Font(filepath(PASS_FONT_NAME),10)
        
        # For angry waiting
        self.foot = 0.0
        self.foot_rate = -0.4
        
        self.lw = BASE_PASS_LINE_WIDTH
        
        # For menu passenger
        self.use_custom_scale = False
        self.custom_scale = getScaling()
        self.footsize = 0.14
        self.doDrawTgt = True
        self.doDrawAnger = True

    def isWaiting(self):
        return (self.status == WAITING or self.status == ANGRYWAITING)

    def isEntering(self):
        return (self.status == ENTERING)

    def isDone(self):
        return (self.status == DONE)

    def isLeaving(self):
        return self.isLeavingAngry() or self.isLeavingHappy()

    def isLeavingAngry(self):
        return (self.patience > ANGRY_LEAVING and \
                (self.status == LEAVING or self.status == DONE))
    
    def isLeavingHappy(self):
        return (self.patience < ANGRY_LEAVING and \
                (self.status == LEAVING or self.status == DONE))

    def isRiding(self):
        return (self.status == RIDING)
        
    def setLeaving(self):
        self.status = LEAVING
        #self.p_rate = 0

    def setRiding(self):
        self.status = RIDING

    def setDrawTgtAndAnger(self,value):
        self.doDrawTgt = value
        self.doDrawAnger = value

    def arrived(self,floor):
        self.status = LEAVING
        self.floor = self.getTgt()
        self.rect.right = floor.getAbsoluteWaitX()
        #self.p_rate = 0

    def getTgt(self):
        return self.tgt

    def setAngry(self):
        self.patience = ANGRY_LEAVING + 1

    def setAngryWaiting(self):
        self.patience = ANGRY_THRESHOLD+1
        self.status = ANGRYWAITING

    def setPatience(self,value):
        self.p_rate = value

    def updatePatience(self):
        if self.status == DONE:
            return
           
        if self.status == WAITING or self.status == ANGRYWAITING:
            self.patience = self.patience + self.p_rate
            if self.patience > ANGRY_THRESHOLD and self.isWaiting():
                self.status = ANGRYWAITING
            if self.patience > ANGRY_LEAVING:
                self.status = LEAVING
                self.patience = ANGRY_LEAVING+1
    
    def getLineWidth(self):
        lw = BASE_PASS_LINE_WIDTH*getScaling()
        if lw <= 0.0:
            lw = 1.0
        return int(lw)
    
    def getFloor(self):
        return self.floor
    
    def setCustomScale(self,value):
        self.use_custom_scale = True
        self.custom_scale = value
    
    def drawMeter(self,surface):
        if not self.doDrawAnger:
            return
        
        rgb = getRGBScaling(self.patience)
        w = int(XMETER_SIZE*surface.get_width())
        x = surface.get_width()-w
        ystart = int(YMETER_SIZE*surface.get_height())
        yend = int((YMETER_SIZE*(100-self.patience)/100.0)*surface.get_height())
        if yend < 0:
            yend = 0
        drect = Rect(x,ystart,w,ystart-yend)
        drect.bottom = ystart
        pygame.draw.rect(surface,rgb,drect,0)
    
    def drawTgt(self,surface):
        if not self.doDrawTgt:
            return
        
        # Target floor
        tgt_floor_ftn = self.font.render(str(self.tgt),1,self.color)
        
        # for debug only...
        #tgt_floor_ftn = self.font.render(str(self.status),1,self.color)
        surface.blit(tgt_floor_ftn,(0,0+tgt_floor_ftn.get_width()))
    
    def drawMeterAndTgt(self,surface):
        self.drawMeter(surface)
        self.drawTgt(surface)
    
    def drawHeadAndTorso(self,surface):
        
        circle_dia = int(0.33*surface.get_height())
        circle_rad = circle_dia/2
        torso = circle_dia
        
        x = surface.get_width()/2
        
        # Draw the head
        pygame.draw.circle(surface,self.color, \
                           (x,circle_rad),circle_rad,self.lw)
        
        # Torso next        self.drawTgt = True
        self.drawAnger = True
        pygame.draw.line(surface,self.color,(x,circle_dia),(x,circle_dia+torso),self.lw)
        
    def checkLegPos(self):
        if self.legpos[0] >= self.rect.width or self.legpos[1] <= 0:
            self.legpos[1] = self.rect.width
            self.legpos[0] = 0
            
    def updateLegs(self,amt):
        self.legpos[0] = self.legpos[0] + amt
        self.legpos[1] = self.legpos[1] - amt
        self.checkLegPos()
    
    def drawWalkingLegs(self,surface):
        
        # Now the legs & arms
        x_start = surface.get_width()/2
        y_start = int(0.67*surface.get_height())
        for x_end in self.legpos:
            pygame.draw.line(surface,self.color,(x_start,y_start),(x_end,surface.get_height()),self.lw)
    
    def setFootRate(self,value):
        self.foot_rate = value
        
    def setFootSize(self,value):
        self.footsize = value
        
    def drawWalking(self,surface):
        self.drawHeadAndTorso(surface)
        
        # Get the line width
        lw = self.getLineWidth()
        
        # Now the legs & arms
        self.drawWalkingLegs(surface)
        x_start = surface.get_width()/2
        y_start = int(0.67*surface.get_height())
        
        y_arm_start = int(0.4*surface.get_height())
        y_arm_end = int(y_arm_start+0.25*surface.get_height())
        
        for x_end in self.legpos:
            arm_x = x_start + int(0.85*float(x_end - x_start))
            pygame.draw.line(surface,self.color,(x_start,y_arm_start),(int(arm_x),y_arm_end),lw)
    
    def drawWaiting(self,surface):
        x_start = surface.get_width()/2
        self.legpos[0] = int(x_start - surface.get_width()/8)
        self.legpos[1] = int(x_start + surface.get_width()/8)
        self.drawWalking(surface)
    
    def drawAngryWaiting(self,surface):
        self.drawHeadAndTorso(surface)
        
        x_start = surface.get_width()/2
        self.legpos[0] = int(x_start - surface.get_width()/8)
        self.legpos[1] = int(x_start + surface.get_width()/8)
        self.drawWalkingLegs(surface)
        
        pygame.draw.line(surface,self.color,(self.legpos[1],self.rect.height-2),(self.rect.width,self.rect.height-int(self.foot)),self.lw)
        
        self.foot = self.foot + self.foot_rate
        if self.foot <= 0.0:
            self.foot = 0.0
            self.foot_rate = -1.0*self.foot_rate
        elif self.foot >= self.footsize*self.rect.height:
            self.foot = self.footsize*self.rect.height
            self.foot_rate = -1.0*self.foot_rate
    
        # Now the legs & arms
        self.drawWalkingLegs(surface)
    
        x_start = surface.get_width()/2
        x_end = int(0.3*surface.get_width())
        y_arm_start = int(0.4*surface.get_height())
        #y_arm_end = int(y_arm_start+0.25*surface.get_height())
        y_arm_end = int(0.6*surface.get_height())
        y_mid = (y_arm_start+y_arm_end)/2
        
        plist = [(x_start,y_arm_start),(x_end,y_mid),(x_start,y_arm_end),(surface.get_width()-x_end,y_mid)]
        pygame.draw.polygon(surface, self.color, plist, self.lw)
    
    def setReachedTgtFloor(self):
        self.floor = self.tgt_floor
    
    def getMoveDirection(self):
        if self.status == ENTERING or self.status == WAITING:
            return 1
        else:
            return -1
    
    def move(self, amt, elevator, floor):
        if self.status == DONE:
            return
        
        if self.isRiding():
            self.rect.bottom = elevator.getElevatorY()
            self.rect.left = elevator.getElevatorX(self)
        else:
            if self.rect.right >= floor.getWaitX(self) and not (self.isRiding() or self.isLeaving()):
                self.status = WAITING
            elif self.rect.left < 0:
                self.status = DONE
            elif self.rect.right < floor.getWaitX(self) and (self.status == WAITING or self.status == ANGRYWAITING):
                self.status = ENTERING
                self.rect.left = self.rect.left + self.getMoveDirection()*amt
                self.updateLegs(amt)
            else:
                self.rect.left = self.rect.left + self.getMoveDirection()*amt
                self.updateLegs(amt)
            
        #if self.rect.right >= floor.getWaitX(self) and not self.isRiding():
        #    self.status = WAITING
        #elif self.rect.left <= 0:
        #    self.status = DONE
            
    def setOnElevator(self,x,elevator):
        self.rect.centerx = x
        self.rect.bottom = elevator.getY()
        self.setRiding()
    
    def createSurface(self):
        # Simple surface creation
        self.image = pygame.Surface((self.rect.width,self.rect.height))
        self.image.convert()
        self.image.set_colorkey(Color('White'))
        self.image.fill(Color('White'))
    
    def updateRect(self):
        scaling = getScaling()
        if self.use_custom_scale:
            scaling = self.custom_scale
        
        if self.rect.height != int(PASS_HEIGHT*scaling):
            scalefactor = float(PASS_HEIGHT)*scaling/float(self.rect.height)
            self.rect.width = self.rect.width*scalefactor
            self.rect.height = self.rect.height*scalefactor
            self.rect.left = self.rect.left*scalefactor


    def update(self):
        
        # Generate the rectangle based on the old
        self.updateRect()
        
        # General line width
        self.lw = self.getLineWidth()
        
        # get the y position
        if not self.isRiding():
            self.rect.bottom = calculateFloorY(self.floor)
        
        # Create the image
        self.createSurface()
    
        # Updates
        self.updatePatience()
    
        # Do painting 
        if self.status == ENTERING:
            self.drawWalking(self.image)
            self.drawMeterAndTgt(self.image)
        elif self.status == LEAVING:
            self.drawWalking(self.image)
            self.drawMeter(self.image)
        elif self.status == WAITING: 
            self.drawWaiting(self.image)
            self.drawMeterAndTgt(self.image)
        elif self.status == RIDING:
            self.drawWaiting(self.image)
        elif self.status == ANGRYWAITING:
            self.drawAngryWaiting(self.image) 
            self.drawMeterAndTgt(self.image)
        elif self.status == RIDING:
            self.drawWaiting(self.image)
        elif self.status == DONE: # Just here as a placeholder
            self.status = DONE
        else:
            print 'Warning - Unhandled passenger status: ',self.status
            self.drawWaiting(self.image)
        
        # If we're using custom scaling
        self.use_custom_scale = False

            
